using System;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;

namespace ASPItalia.Pubs.Business
{
    public class AuthorManager
    {
        private readonly bool _cacheEnabled = ConfigurationService.CacheEnabled;

        public AuthorManager()
        {
        }

        public AuthorManager(bool cacheEnabled)
        {
            _cacheEnabled = cacheEnabled;
        }

        public Author GetSingle(string authorID)
        {
            if (string.IsNullOrEmpty(authorID))
                throw new ArgumentException("Invalid author ID");

            return ServiceFactory.Instance.CreateAuthorService(_cacheEnabled).ReadSingle(authorID);
        }

        public List<Author> GetList()
        {
            return ServiceFactory.Instance.CreateAuthorService(_cacheEnabled).ReadAll() as List<Author>;
        }

        public void Create(Author author)
        {
            if ((author != null) && (!string.IsNullOrEmpty(author.AuthorID)))
                ServiceFactory.Instance.CreateAuthorService(_cacheEnabled).Create(author);
            else
                throw new ArgumentException("Invalid author");
        }

        public void Update(Author author)
        {
            if ((author != null) && (!string.IsNullOrEmpty(author.AuthorID)))
                ServiceFactory.Instance.CreateAuthorService(_cacheEnabled).Update(author);
            else
                throw new ArgumentException("Invalid author");
        }

        public void Delete(string authorID)
        {
            if (string.IsNullOrEmpty(authorID))
                throw new ArgumentException("Invalid author ID");

            ServiceFactory.Instance.CreateAuthorService(_cacheEnabled).Delete(authorID);
        }
    }
}
