using System;
using System.Linq;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;

namespace ASPItalia.Pubs.Business
{
    public class BookManager
    {
        private readonly bool _cacheEnabled = ConfigurationService.CacheEnabled;

        public BookManager()
        {
        }

        public BookManager(bool cacheEnabled)
        {
            _cacheEnabled = cacheEnabled;
        }

        public Book GetSingle(string bookID)
        {
            if (string.IsNullOrEmpty(bookID))
                throw new ArgumentException("Invalid book ID");

            return ServiceFactory.Instance.CreateBookService(_cacheEnabled).ReadSingle(bookID);
        }

        public List<Book> GetList()
        {
            return ServiceFactory.Instance.CreateBookService(_cacheEnabled).ReadAll() as List<Book>;
        }

        public List<Book> GetListByAuthor(string authorID)
        {
            if (string.IsNullOrEmpty(authorID))
                return new List<Book>();

            var books = (from b in GetList()
                         from a in b.Authors
                         where a.Author.AuthorID.Equals(authorID)
                         select a.Book)
                        .ToList<Book>();

            return books;
        }

        public List<Book> GetListByPublisher(string publisherID)
        {
            if (string.IsNullOrEmpty(publisherID))
                return new List<Book>();

            return GetList().Where<Book>(b => b.Publisher.PublisherID.Equals(publisherID)).ToList<Book>();
        }

        public List<Book> GetListByCategory(string category)
        {
            if (string.IsNullOrEmpty(category))
                return new List<Book>();

            return GetList().Where<Book>(b => b.Category.Equals(category)).ToList<Book>();
        }

        public void Create(Book book)
        {
            if ((book != null) && (!string.IsNullOrEmpty(book.BookID)) && ((book.Authors != null) && (book.Authors.Count > 0)) && ((book.Publisher != null) && (!string.IsNullOrEmpty(book.Publisher.PublisherID))))
                ServiceFactory.Instance.CreateBookService(_cacheEnabled).Create(book);
            else
                throw new ArgumentException("Invalid book");
        }

        public void Update(Book book)
        {
            if ((book != null) && (!string.IsNullOrEmpty(book.BookID)) && ((book.Authors != null) && (book.Authors.Count > 0)) && ((book.Publisher != null) && (!string.IsNullOrEmpty(book.Publisher.PublisherID))))
                ServiceFactory.Instance.CreateBookService(_cacheEnabled).Update(book);
            else
                throw new ArgumentException("Invalid book");
        }

        public void Delete (string bookID)
        {
            if (string.IsNullOrEmpty(bookID))
                throw new ArgumentException("Invalid book ID");

            ServiceFactory.Instance.CreateBookService(_cacheEnabled).Delete(bookID);
        }
    }
}
