using System;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;
using ASPItalia.Pubs.Business;

namespace ASPItalia.Pubs.Business.Caching
{
    internal class AuthorCacheService : IEntityService<Author>
    {
        private readonly AuthorService _target = new AuthorService();

        public Author ReadSingle(string authorID)
        {
            Author author = null;

            if (CachingService.Get("author-" + authorID) == null)
            {
                author = _target.ReadSingle(authorID);
                CachingService.Insert("author-" + authorID, author);
            }
            else
            {
                author = CachingService.Get("author-" + authorID) as Author;
            }

            return author;
        }

        public List<Author> ReadAll()
        {
            List<Author> authors = null;

            if (CachingService.Get("authors") == null)
            {
                authors = _target.ReadAll();
                CachingService.Insert("authors", authors);
            }
            else
            {
                authors = CachingService.Get("authors") as List<Author>;
            }

            return authors;
        }

        public void Create(Author author)
        {
            _target.Create(author);
            CachingService.Remove("authors");
            CachingService.Remove("author-" + author.AuthorID);
        }

        public void Update(Author author)
        {
            _target.Update(author);
            CachingService.Remove("authors");
            CachingService.Remove("author-" + author.AuthorID);
        }

        public void Delete(string authorID)
        {
            _target.Delete(authorID);
            CachingService.Remove("authors");
            CachingService.Remove("author-" + authorID);
        }

        Author IEntityService<Author>.ReadSingle(string entityID)
        {
            return this.ReadSingle(entityID);
        }

        List<Author> IEntityService<Author>.ReadAll()
        {
            return this.ReadAll();
        }

        void IEntityService<Author>.Create(Author entity)
        {
            this.Create(entity);
        }

        void IEntityService<Author>.Update(Author entity)
        {
            this.Update(entity);
        }

        void IEntityService<Author>.Delete(string entityID)
        {
            this.Delete(entityID);
        }
    }
}
