using System;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;
using ASPItalia.Pubs.Business;

namespace ASPItalia.Pubs.Business.Caching
{
    internal class BookCacheService : IEntityService<Book>
    {
        private readonly BookService _target = new BookService();

        public Book ReadSingle(string bookID)
        {
            Book book = null;

            if (CachingService.Get("book-" + bookID) == null)
            {
                book = _target.ReadSingle(bookID);
                CachingService.Insert("book-" + bookID, book);
            }
            else
            {
                book = CachingService.Get("book-" + bookID) as Book;
            }

            return book;
        }

        public List<Book> ReadAll()
        {
            List<Book> books = null;

            if (CachingService.Get("books") == null)
            {
                books = _target.ReadAll();
                CachingService.Insert("books", books);
            }
            else
            {
                books = CachingService.Get("books") as List<Book>;
            }

            return books;
        }

        public void Create(Book book)
        {
            _target.Create(book);
            CachingService.Remove("books");
            CachingService.Remove("book-" + book.BookID);
        }

        public void Update(Book book)
        {
            _target.Update(book);
            CachingService.Remove("books");
            CachingService.Remove("book-" + book.BookID);
        }

        public void Delete(string bookID)
        {
            _target.Delete(bookID);
            CachingService.Remove("books");
            CachingService.Remove("book-" + bookID);
        }

        Book IEntityService<Book>.ReadSingle(string entityID)
        {
            return this.ReadSingle(entityID);
        }

        List<Book> IEntityService<Book>.ReadAll()
        {
            return this.ReadAll();
        }

        void IEntityService<Book>.Create(Book entity)
        {
            this.Create(entity);
        }

        void IEntityService<Book>.Update(Book entity)
        {
            this.Update(entity);
        }

        void IEntityService<Book>.Delete(string entityID)
        {
            this.Delete(entityID);
        }
    }
}
