using System;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;
using ASPItalia.Pubs.Business;

namespace ASPItalia.Pubs.Business.Caching
{
    internal class PublisherCacheService : IEntityService<Publisher>
    {
        private readonly PublisherService _target = new PublisherService();

        public Publisher ReadSingle(string publisherID)
        {
            Publisher publisher = null;

            if (CachingService.Get("publisher-" + publisherID) == null)
            {
                publisher = new PublisherService().ReadSingle(publisherID);
                CachingService.Insert("publisher-" + publisherID, publisher);
            }
            else
            {
                publisher = CachingService.Get("publisher-" + publisherID) as Publisher;
            }

            return publisher;
        }

        public List<Publisher> ReadAll()
        {
            List<Publisher> publishers = null;

            if (CachingService.Get("publishers") == null)
            {
                publishers = new PublisherService().ReadAll();
                CachingService.Insert("publishers", publishers);
            }
            else
            {
                publishers = CachingService.Get("publishers") as List<Publisher>;
            }

            return publishers;
        }

        public void Create(Publisher publisher)
        {
            _target.Create(publisher);
            CachingService.Remove("publishers");
            CachingService.Remove("publisher-" + publisher.PublisherID);
        }

        public void Update(Publisher publisher)
        {
            _target.Update(publisher);
            CachingService.Remove("publishers");
            CachingService.Remove("publisher-" + publisher.PublisherID);
        }

        public void Delete(string publisherID)
        {
            _target.Delete(publisherID);
            CachingService.Remove("publishers");
            CachingService.Remove("publisher-" + publisherID);
        }

        Publisher IEntityService<Publisher>.ReadSingle(string entityID)
        {
            return this.ReadSingle(entityID);
        }

        List<Publisher> IEntityService<Publisher>.ReadAll()
        {
            return this.ReadAll();
        }

        void IEntityService<Publisher>.Create(Publisher entity)
        {
            this.Create(entity);
        }

        void IEntityService<Publisher>.Update(Publisher entity)
        {
            this.Update(entity);
        }

        void IEntityService<Publisher>.Delete(string entityID)
        {
            this.Delete(entityID);
        }
    }
}
