using System;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;

namespace ASPItalia.Pubs.Business
{
    public class PublisherManager
    {
        private readonly bool _cacheEnabled = ConfigurationService.CacheEnabled;

        public PublisherManager()
        {
        }

        public PublisherManager(bool cacheEnabled)
        {
            _cacheEnabled = cacheEnabled;
        }

        public Publisher GetSingle(string publisherID)
        {
            if (string.IsNullOrEmpty(publisherID))
                throw new ArgumentException("Invalid publisher ID");

            return ServiceFactory.Instance.CreatePublisherService(_cacheEnabled).ReadSingle(publisherID);
        }

        public List<Publisher> GetList()
        {
            return ServiceFactory.Instance.CreatePublisherService(_cacheEnabled).ReadAll();
        }

        public void Create(Publisher publisher)
        {
            if ((publisher != null) && (!string.IsNullOrEmpty(publisher.PublisherID)))
                ServiceFactory.Instance.CreatePublisherService(_cacheEnabled).Create(publisher);
            else
                throw new ArgumentException("Invalid publisher ID");
        }

        public void Update(Publisher publisher)
        {
            if ((publisher != null) && (!string.IsNullOrEmpty(publisher.PublisherID)))
                ServiceFactory.Instance.CreatePublisherService(_cacheEnabled).Update(publisher);
            else
                throw new ArgumentException("Invalid publisher ID");
        }

        public void Delete(string publisherID)
        {
            if (string.IsNullOrEmpty(publisherID))
                throw new ArgumentException("Invalid publisher ID");

            ServiceFactory.Instance.CreatePublisherService(_cacheEnabled).Delete(publisherID);
        }
    }
}
