using System;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;
using ASPItalia.Pubs.Business.Caching;

namespace ASPItalia.Pubs.Business
{
    internal sealed class ServiceFactory : IServiceFactory
    {
        public static readonly IServiceFactory Instance = new ServiceFactory();

        private ServiceFactory()
        {
        }

        public IEntityService<Book> CreateBookService(bool cacheEnabled)
        {
            IEntityService<Book> service = null;

            if (cacheEnabled && ConfigurationService.CacheEnabled)
                service = new BookCacheService();
            else
                service = new BookService();

            return service;
        }

        public IEntityService<Author> CreateAuthorService(bool cacheEnabled)
        {
            IEntityService<Author> service = null;

            if (cacheEnabled && ConfigurationService.CacheEnabled)
                service = new AuthorCacheService();
            else
                service = new AuthorService();

            return service;
        }

        public IEntityService<Publisher> CreatePublisherService(bool cacheEnabled)
        {
            IEntityService<Publisher> service = null;

            if (cacheEnabled && ConfigurationService.CacheEnabled)
                service = new PublisherCacheService();
            else
                service = new PublisherService();

            return service;
        }
    }
}
