using System;
using System.Web;
using System.Web.Caching;
using System.Collections;
using System.Threading;

namespace ASPItalia.Pubs.Common.Services
{
    public static class CachingService
    {
        private static readonly Cache _cacheInternal = HttpContext.Current.Cache;

        public static void Insert(string key, object value)
        {
            Insert(key, value, ConfigurationService.DefaultCacheTimeout);
        }

        public static void Insert(string key, object value, int timeout)
        {
            if (string.IsNullOrEmpty(key))
                throw new ArgumentException("Invalid key.");

            if (timeout <= 0)
                timeout = ConfigurationService.DefaultCacheTimeout;

            CacheFileService service = new CacheFileService(key + ".cache");
            service.CreateFile();

            _cacheInternal.Insert(
                key,
                value,
                new CacheDependency(service.FullPath),
                DateTime.Now.AddMinutes(timeout),
                TimeSpan.Zero,
                CacheItemPriority.Default,
                OnRemoveCacheItem);
        }

        public static void Remove(string key)
        {
            if (!string.IsNullOrEmpty(key))
                _cacheInternal.Remove(key);
        }

        public static object Get(string key)
        {
            if (!string.IsNullOrEmpty(key))
                return _cacheInternal[key];
            return null;
        }

        public static void Clear()
        {
            CacheFileService.Clear();
        }

        private static void OnRemoveCacheItem(string key, object value, CacheItemRemovedReason reason)
        {
            CacheFileService service = new CacheFileService(key + ".cache");
            service.DeleteFile();
        }
    }
}