﻿using System;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;

namespace ASPItalia.Pubs.Data
{
    internal sealed class AuthorData
    {
        private readonly PubsDataContext _context;

        public AuthorData(PubsDataContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            _context = context;
        }

        public List<Author> GetAuthorList()
        {
            var authors =
                (from a in _context.Authors
                 orderby a.LastName, a.FirstName
                 select a)
                .ToList<Author>();

            return authors;
        }

        public Author GetAuthor(string authorID)
        {
            if (string.IsNullOrEmpty(authorID))
                return new Author();

            var author = _context.Authors.SingleOrDefault(a => a.AuthorID.Equals(authorID));
            return author ?? new Author();
        }

        public void CreateAuthor(Author author)
        {
            if (author == null)
                throw new ArgumentNullException("author");

            if (!string.IsNullOrEmpty(author.AuthorID))
                _context.Authors.InsertOnSubmit(author);
        }

        public void UpdateAuthor(Author author)
        {
            if (author == null)
                throw new ArgumentNullException("author");

            if (!string.IsNullOrEmpty(author.AuthorID))
            {
                Author a = GetAuthor(author.AuthorID);
                if (!string.IsNullOrEmpty(a.AuthorID))
                {
                    a.FirstName = author.FirstName;
                    a.LastName = author.LastName;
                    a.PhoneNumber = author.PhoneNumber;
                    a.HasContract = author.HasContract;
                }
            }
        }

        public void DeleteAuthor(string authorID)
        {
            if (string.IsNullOrEmpty(authorID))
                throw new ArgumentException("Invalid author ID");

            Author author = GetAuthor(authorID);

            if (author != null)
                _context.Authors.DeleteOnSubmit(author);
        }
    }
}
