﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using ASPItalia.Pubs.Common.ObjectModel;
using ASPItalia.Pubs.Common.Services;
using System.IO;

namespace ASPItalia.Pubs.Data
{
    public class DataService : IDisposable
    {
        private readonly PubsDataContext _context;
        private readonly AuthorData _authors;
        private readonly BookData _books;
        private readonly PublisherData _publishers;
        private readonly StreamWriter _writer;

        public DataService() : this(false)
        {
        }

        public DataService(bool isReadOnly)
        {
            _context = new PubsDataContext(ConfigurationService.ConnectionString);
            _context.ObjectTrackingEnabled = !isReadOnly;

            if (ConfigurationService.QueryLogEnabled)
            {
                _writer = new StreamWriter(ConfigurationService.QueryLogFilename, true);
                _context.Log = _writer;
            }

            _authors = new AuthorData(_context);
            _books = new BookData(_context);
            _publishers = new PublisherData(_context);
        }

        public List<Author> GetAuthorList()
        {
            return _authors.GetAuthorList();
        }

        public List<Book> GetBookList()
        {
            return _books.GetBookList();
        }
        
        public List<Publisher> GetPublisherList()
        {
            return _publishers.GetPublisherList();
        }

        public Author GetAuthor(string authorID)
        {
            return _authors.GetAuthor(authorID);
        }

        public Book GetBook(string bookID)
        {
            return _books.GetBook(bookID);
        }

        public Publisher GetPublisher(string publisherID)
        {
            return _publishers.GetPublisher(publisherID);
        }

        public void CreateAuthor(Author author)
        {
            _authors.CreateAuthor(author);
        }

        public void CreateBook(Book book)
        {
            _books.CreateBook(book);
        }

        public void CreatePublisher(Publisher publisher)
        {
            _publishers.CreatePublisher(publisher);
        }

        public void UpdateAuthor(Author author)
        {
            _authors.UpdateAuthor(author);
        }

        public void UpdateBook(Book book)
        {
            _books.UpdateBook(book);
        }

        public void UpdatePublisher(Publisher publisher)
        {
            _publishers.UpdatePublisher(publisher);
        }

        public void DeleteAuthor(string authorID)
        {
            _authors.DeleteAuthor(authorID);
        }

        public void DeleteBook(string bookID)
        {
            _books.DeleteBook(bookID);
        }

        public void DeletePublisher(string publisherID)
        {
            _publishers.DeletePublisher(publisherID);
        }

        public void Dispose()
        {
            try
            {
                if (_context.ObjectTrackingEnabled)
                    _context.SubmitChanges();

                if (_writer != null)
                    _writer.Flush();
            }
            catch (Exception ex)
            {
                // throw ex;
            }
            finally
            {
                _context.Dispose();

                if (_writer != null)
                    _writer.Dispose();
            }
        }
    }
}
