﻿using System;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Collections.Generic;
using ASPItalia.Pubs.Common.ObjectModel;

namespace ASPItalia.Pubs.Data
{
    internal sealed class PublisherData
    {
        private readonly PubsDataContext _context;

        public PublisherData(PubsDataContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            _context = context;
        }

        public List<Publisher> GetPublisherList()
        {
            var publishers =
                (from p in _context.Publishers
                 orderby p.Name
                 select p)
                .ToList<Publisher>();

            return publishers;
        }

        public Publisher GetPublisher(string publisherID)
        {
            if(string.IsNullOrEmpty(publisherID))
                return new Publisher();

            var publisher = _context.Publishers.SingleOrDefault(p => p.PublisherID.Equals(publisherID));
            return publisher ?? new Publisher();
        }

        public void CreatePublisher(Publisher publisher)
        {
            if (publisher == null)
                throw new ArgumentNullException("publisher");

            if (!string.IsNullOrEmpty(publisher.PublisherID))
                _context.Publishers.InsertOnSubmit(publisher);
        }

        public void UpdatePublisher(Publisher publisher)
        {
            if (publisher == null)
                throw new ArgumentNullException("publisher");

            if (!string.IsNullOrEmpty(publisher.PublisherID))
            {
                Publisher p = GetPublisher(publisher.PublisherID);
                if (!string.IsNullOrEmpty(p.PublisherID))
                {
                    p.Name = publisher.Name;
                }
            }
        }

        public void DeletePublisher(string publisherID)
        {
            if (string.IsNullOrEmpty(publisherID))
                throw new ArgumentException("Invalid publisher ID");

            Publisher publisher = GetPublisher(publisherID);

            if(publisher != null)
                _context.Publishers.DeleteOnSubmit(publisher);
        }
    }
}
